import datetime as dt
import json
from dataclasses import dataclass
from enum import IntEnum
from typing import Any

import requests

from notifier.notify.notification import Notification, NotificationSender, MobileOS


class NtfyPriority(IntEnum):
    MIN = 1
    LOW = 2
    DEFAULT = 3
    HIGH = 4
    MAX = 5


@dataclass
class NtfyNotification(Notification):
    priority: int = NtfyPriority.DEFAULT

    def generate_payload(self, mobile_os: MobileOS, dad: bool = False) -> dict[str, Any]:
        match mobile_os, dad:
            case _, True:
                topic = "frigate_notifications_dad"
            case MobileOS.ANDROID, False:
                topic = "frigate_notifications"
            case MobileOS.IOS, False:
                topic = "frigate_notifications_ios"

        return {
            "topic": topic,
            "title": "Frigate",
            "message": f"{self.object_label.capitalize()} at {self.camera_display_name} ({self.score:.0%})",
            "priority": NtfyPriority.DEFAULT,
            "click": self.camera_url,
            "icon": "https://cdn.jsdelivr.net/gh/walkxcode/dashboard-icons/png/frigate.png",
            "attach": self.get_image_url(mobile_os),
            "actions": [
                {
                    "action": "http",
                    "label": "DBL (30m)" if dad else "Disable (30m)",
                    "url": f"https://frigate.homelab.net/webcontrol/camera/{self.camera}/detection",
                    "method": "POST",
                    "headers": {
                        "Content-Type": "application/json"
                    },
                    "body": json.dumps({
                        "detection": False,
                        "duration": 30
                    }),
                    "clear": not dad,
                },
            ],
        }


class NtfyNotificationSender(NotificationSender[NtfyNotification]):
    def __init__(self, quiet_period: dt.timedelta, base_url: str = "https://ntfy.homelab.net") -> None:
        super().__init__(quiet_period)
        self._base_url = base_url

    def _send(self, notification: NtfyNotification) -> None:
        # Send to Android topic
        payload = notification.generate_payload(MobileOS.ANDROID)
        response = requests.post(self._base_url, json=payload)
        response.raise_for_status()

        # Send to iOS topic
        payload = notification.generate_payload(MobileOS.IOS)
        response = requests.post(self._base_url, json=payload)
        response.raise_for_status()

        # Send to Dad topic
        payload = notification.generate_payload(MobileOS.ANDROID, dad=True)
        response = requests.post(self._base_url, json=payload)
        response.raise_for_status()
