import datetime as dt
import json
import logging
import os
import urllib.parse
from collections.abc import AsyncGenerator

import aiohttp

logger = logging.getLogger(__name__)

SUNSET_FALLBACK = dt.time(19, 0, 0)
DARKNESS_DELAY = dt.timedelta(minutes=30)


async def get_nighttimes(location: str) -> AsyncGenerator[dt.datetime, None]:
    async with aiohttp.ClientSession(raise_for_status=True) as aiohttp_session:
        date = dt.date.today()
        sunset_time = SUNSET_FALLBACK

        while True:
            try:
                ipgeolocation_api_url = "https://api.ipgeolocation.io/astronomy?apiKey={api_key}&location={location}&date={date}".format(
                    api_key=urllib.parse.quote(os.environ["IPGEOLOCATION_API_KEY"]),
                    location=urllib.parse.quote(location),
                    date=urllib.parse.quote(date.strftime("%Y-%m-%d"))
                )
                async with aiohttp_session.get(ipgeolocation_api_url) as response:
                    response_json = json.loads(await response.read())
                    sunset_time = dt.datetime.strptime(response_json["sunset"], "%H:%M").time()
            except Exception as e:
                logger.error(f"Failed to query sunset time, falling back to {sunset_time}", exc_info=e)

            nighttime = dt.datetime.combine(date, sunset_time) + DARKNESS_DELAY
            date += dt.timedelta(days=1)
            if nighttime < dt.datetime.now():
                continue
            yield nighttime

